#include "Keyboard.h"

#define NUM_BTN_COLS (5)
#define NUM_BTN_ROWS (4)
#define MAX_DEBOUNCE (3)

static const uint8_t btnRowPins[NUM_BTN_ROWS] = {2, 3, 4, 5};
static const uint8_t btnColPins[NUM_BTN_COLS] = {A3, A2, A1, 13, 12};
static int8_t debounce_count[NUM_BTN_COLS][NUM_BTN_ROWS];

int teclas[] = {'1', '2', '3', '4', '5', KEY_TAB, 'q', 'w', 'e', 'r', KEY_LEFT_CTRL, 'd', 'f', 'b', ' ', 'z', 'x', 'c', 'v', KEY_ESC};

//LEDS
byte leds1 = 0;//por shift register 1
byte leds2 = 0;//por shift register 2

//shift register
//latch pin (ST_CP) 74HC595
const int latchPin = 6;
//clock pin (SH_CP) 74HC595
const int clockPin = 7;
//Data pin (DS) 74HC595
const int dataPin = 8;

void setup() {
  Serial.begin(115200);
  setupLEDPins();
  setupSwitchPins();
  for(uint8_t i = 0; i < NUM_BTN_COLS; i++) {
    for(uint8_t j = 0; j < NUM_BTN_ROWS; j++) {
      debounce_count[i][j] = 0;
    }
  }
  Keyboard.begin();
}

void loop() {
  scan();
}

static void scan() {
  static uint8_t currentRow = 0;
  uint8_t i, j;

  digitalWrite(btnRowPins[currentRow], LOW);

  for(j = 0; j < NUM_BTN_COLS; j++) {
    if(digitalRead(btnColPins[j]) == LOW) {
      if(debounce_count[currentRow][j] < MAX_DEBOUNCE) {
        debounce_count[currentRow][j]++;
        if(debounce_count[currentRow][j] == MAX_DEBOUNCE) {
          tecla((currentRow * NUM_BTN_COLS) + j, 1, 0);
          //Serial.print("Key pressed "); 
          //Serial.println((currentRow * NUM_BTN_COLS) + j);
        }
      }
    } else {
      if(debounce_count[currentRow][j] > 0) {
        debounce_count[currentRow][j]--;
        if(debounce_count[currentRow][j] == 0) {
          tecla((currentRow * NUM_BTN_COLS) + j, 0, 1);
          //Serial.print("Key released ");
          //Serial.println((currentRow * NUM_BTN_COLS) + j);
        }
      }
    }
  }
  digitalWrite(btnRowPins[currentRow], HIGH);
  currentRow++;
  if(currentRow >= NUM_BTN_ROWS) {
    currentRow = 0;
  }
}

static void setupLEDPins() {
  pinMode(latchPin, OUTPUT);
  pinMode(dataPin, OUTPUT);  
  pinMode(clockPin, OUTPUT);
}

static void setupSwitchPins() {
  uint8_t i;
  for(i = 0; i < NUM_BTN_ROWS; i++) {
    pinMode(btnRowPins[i], OUTPUT);
    digitalWrite(btnRowPins[i], HIGH);
  }
  for(i = 0; i < NUM_BTN_COLS; i++) {
    pinMode(btnColPins[i], INPUT_PULLUP);
  }
}

void led(int num, int status) {
  if(num < 8) {//es con shift register 1
    if(status == 0) {
      bitClear(leds1, num);
    } else {
      bitSet(leds1, num);
    }
  } else {//es con shift register 2
    if(status == 0) {
      bitClear(leds2, num-8);
    } else {
      bitSet(leds2, num-8);
    }
  }
}
void leds() {
  //por shift register
  digitalWrite(latchPin, LOW);
  shiftOut(dataPin, clockPin, MSBFIRST, leds1);
  shiftOut(dataPin, clockPin, MSBFIRST, leds2);
  digitalWrite(latchPin, HIGH);
}
void tecla(int num, int status, int status_led) {
  if(status == 1) {
    Keyboard.press(teclas[num]);
    led(num, status_led);
  } else {
    Keyboard.release(teclas[num]);
    led(num, status_led);
  }
}
